using System;
using Microsoft.Win32;
using System.Net;
using System.Drawing;
using System.Text;
using System.Runtime.InteropServices;
using Amib.Threading;

namespace CustomBandObjectLib
{
	/// <summary>
	/// Summary description for common.
	/// </summary>
	public class Common
	{
		string searchString = "";
		public Log Logger;
		public RegistryHandler Reg;
		public SmartThreadPool Pool;
		public ResponseParser Parser;
		public FavoritesHandler FavHandler;
		public string LocationURL; 

		public Common()
		{
			Logger = new Log();
			Reg = new RegistryHandler();
			Pool = new SmartThreadPool();
			Parser = new ResponseParser();
			FavHandler = new FavoritesHandler(Reg);
		}

		[DllImport("wininet.dll", CharSet=CharSet.Auto, SetLastError=true)]
		public static extern bool InternetGetCookie(
			string lpszUrlName,
			string lpszCookieName,
			StringBuilder lpszCookieData,
			[MarshalAs(UnmanagedType.U4)]
			ref int lpdwSize
			);

		public static CookieContainer GetUriCookieContainer(Uri uri) 
		{ 
			CookieContainer cookies = null;
  
			int datasize = 256;
			StringBuilder cookieData = new StringBuilder(datasize);

			if (!InternetGetCookie(uri.ToString(), null, cookieData, 
				ref datasize)) 
			{
				if (datasize < 0) 
					return null; 

				cookieData = new StringBuilder(datasize);
				if ( !InternetGetCookie(uri.ToString(), null, cookieData, 
					ref datasize) )
					return null;
			} 


			if (cookieData.Length > 0)
			{
				cookies = new CookieContainer();
				cookies.SetCookies(uri, cookieData.ToString().Replace(';', ','));
			}
			return cookies;
		}

		public string[] GetTriggersArray(Uri uri)
		{
			searchString = this.FavHandler.GetFavoriteTrigger(uri.Host.ToString	());
			char[] sep = new char[1];
			sep[0] = ';';
			string[] subs = null;
			if (searchString != null)
			{
				subs = searchString.Split(sep);
			}	
			return subs;
		}

		public string BuildRegExString(Uri uri)
		{

			string[] aTriggers = null;
			try
			{
				aTriggers = GetTriggersArray(uri);
			}
			catch {}
			System.Text.StringBuilder sb = new System.Text.StringBuilder("");

			if (aTriggers != null)
			{
				if (aTriggers[0] != "")
				{ 
					for (int i = 0;i < aTriggers.Length; i++)
					{
						sb.Append("\\"+ aTriggers[i] +"|");
					}
				}
			}

			return sb.ToString().TrimEnd('|');
		}
		
		protected object CheckLink(object oURL)
		{
			string sURL = (string)oURL;

			HttpWebRequest request = (HttpWebRequest) WebRequest.Create(sURL);
			WebResponse response = request.GetResponse();
			if (response.ContentType == "application/x-bittorrent")
			{
				return true; 
			}
			else
			{
				request = (HttpWebRequest) WebRequest.Create(sURL);
				if(this.FavHandler.GetFavoriteCookie(new Uri(sURL).Host))
				{
					request.CookieContainer = GetUriCookieContainer(new Uri(sURL));
				}
				response = request.GetResponse();
				if (response.ContentType == "application/x-bittorrent")
				{
					return true; 
				}
			}

			return false;
		}
	}	 
}
