using System;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Reflection;
using System.Drawing;
using System.ComponentModel;
using Microsoft.Win32;
using BandObjectLib;

[assembly: AssemblyVersion("1.0.0.0")]
[assembly: AssemblyKeyFile(@"..\..\CustomBandObjects.snk")]

namespace CustomBandObjectLib
{
	/// <summary>
	/// 
	/// </summary>
	public class CustomBandObject : BandObjectLib.BandObject 
	{
		public MyWebBrowser	IE;
		public PostThread Post;
		public Common oCommon;

		public delegate void ActionDelegate(int action, string objectID);
		public delegate void SpecialFileDownLoadEventHandler(SpecialFileDownloadEventArgs e);
		public delegate void XmlChangeDelegate(String xml);
		public delegate void SpecialFileDownloadDelegate(string url);
		public event XmlChangeDelegate XmlChangeEvent;
		public event ActionDelegate ActionEvent;
		public event SpecialFileDownloadDelegate SpecialFileDownloadEvent;

		public CustomBandObject()
		{
			InitializeComponent();
		}

		private void InitializeComponent()
		{
			this.BackColor = System.Drawing.SystemColors.Control;
			this.Name = "BandObject";
		}

		/// <summary>
		/// 
		/// </summary>
		[Browsable(true)]
		[DefaultValue("")]
		public override Color BackColor
		{
			get
			{
				return _backColor;
			}
			set
			{
				_backColor = value;
			}
		}Color _backColor;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="fShow"></param>
		public override void ShowDW(bool fShow)
		{
			if( fShow )
			{
				if (Post == null)
				{
					Show();
					if (!IE.IsAdvised())
					{
						IE.Advise();
					}
					((IEEvents)IE.Sink).SpecialFileDownLoadEvent +=new CustomBandObjectLib.IEEvents.SpecialFileDownloadDelegate(BandObject_SpecialFileDownLoadEvent);
					Post = new PostThread(this,oCommon);
					Post.XmlChangeEvent +=new CustomBandObjectLib.PostThread.XmlChangeDelegate(post_XmlChangeEvent);
				}
			}
			else
			{
				Hide();
				IE.Unadvise();
				Post = null;				
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="dwReserved"></param>
		public override void CloseDW(UInt32  dwReserved)
		{
			Dispose( true );
			IE.Unadvise();
			oCommon.Pool.Shutdown();
			oCommon = null;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="pUnkSite"></param>
		public override void SetSite(Object pUnkSite)
		{
			if( BandObjectSite != null )
				Marshal.ReleaseComObject( BandObjectSite );

			if( Explorer != null )
			{
				Marshal.ReleaseComObject( Explorer );
				Explorer = null;
			}

			BandObjectSite = (IInputObjectSite)pUnkSite;
			if( BandObjectSite != null )
			{
				_IServiceProvider sp = BandObjectSite as _IServiceProvider;
				Guid guid = ExplorerGUIDs.IID_IWebBrowserApp;
				Guid riid = ExplorerGUIDs.IID_IUnknown;
				
				try
				{
					object w;
					sp.QueryService( 
						ref guid,
						ref riid,
						out w );
				
					IE = (MyWebBrowser)Marshal.CreateWrapperOfType(
						w as SHDocVw.InternetExplorer,
						typeof(MyWebBrowser)
						);

					OnExplorerAttached(EventArgs.Empty);
				}
				catch( COMException )
				{
				}
			}

		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="ea"></param>
		protected override void OnExplorerAttached(EventArgs ea)
		{
			oCommon = new Common();
			base.OnExplorerAttached(ea);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="e"></param>
		private void BandObject_SpecialFileDownLoadEvent(SpecialFileDownloadEventArgs e)
		{
			SpecialFileDownloadEvent(e.Url);	
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="xml"></param>
		private void post_XmlChangeEvent(String xml)
		{
			XmlChangeEvent(xml);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="action"></param>
		/// <param name="objectID"></param>
		public void Action(int action, string objectID)
		{
			ActionEvent(action, objectID);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="URL"></param>
		public void DoNavigate(object URL)
		{
			object url = URL;
			object Flags = null;
			object TargetFrameName = null;
			object PostData = null;
			object Headers = null;

			this.IE.Navigate2 (ref url, ref Flags, ref TargetFrameName, ref PostData, ref Headers);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="t"></param>
		[ComRegisterFunctionAttribute]
		public new static void Register(Type t)
		{
			string guid = t.GUID.ToString("B");
			
			RegistryKey rkClass = Registry.ClassesRoot.CreateSubKey(@"CLSID\"+guid );
			RegistryKey rkCat = rkClass.CreateSubKey("Implemented Categories");

			BandObjectAttribute[] boa = (BandObjectAttribute[])t.GetCustomAttributes(
				typeof(BandObjectAttribute),
				false );

			string name = t.Name;
			string help = t.Name;
			BandObjectStyle style = 0;
			if( boa.Length == 1 )
			{
				if( boa[0].Name != null )
					name = boa[0].Name;

				if( boa[0].HelpText != null )
					help = boa[0].HelpText;

				style = boa[0].Style;
			}

			rkClass.SetValue(null, name );
			rkClass.SetValue("MenuText", name );
			rkClass.SetValue("HelpText", help );

			if( 0 != (style & BandObjectStyle.Horizontal) )
				rkCat.CreateSubKey("{00021494-0000-0000-C000-000000000046}");
		}
   
		/// <summary>
		/// 
		/// </summary>
		/// <param name="t"></param>
		[ComUnregisterFunctionAttribute]
		public new static void Unregister(Type t)
		{
			string guid = t.GUID.ToString("B");
			BandObjectAttribute[] boa = (BandObjectAttribute[])t.GetCustomAttributes(
				typeof(BandObjectAttribute),
				false );

			BandObjectStyle style = 0;
			if( boa.Length == 1 ) style = boa[0].Style;

			if( 0 != (style & BandObjectStyle.ExplorerToolbar) )
				Registry.LocalMachine.CreateSubKey(@"SOFTWARE\Microsoft\Internet Explorer\Toolbar").DeleteValue(guid,false);

			Registry.ClassesRoot.CreateSubKey(@"CLSID").DeleteSubKeyTree(guid);
		}
		
	}

	/// <summary>
	/// 
	/// </summary>
	public abstract class ExplorerGUIDs
	{
		public static readonly Guid IID_IWebBrowserApp = new Guid("{0002DF05-0000-0000-C000-000000000046}");
		public static readonly Guid IID_IUnknown =		 new Guid("{00000000-0000-0000-C000-000000000046}");
	}
}