using System;
using System.Net;
using System.IO;
using System.Threading;
using System.Text;
using System.Diagnostics;
using BandObjectLib;

namespace CustomBandObjectLib
{
	/// <summary>
	/// Summary description for PostThread.
	/// </summary>
	public class PostThread:Protocol
	{
		private ResponseObject obj = null;
		public delegate void XmlChangeDelegate(String xml); 
		public event XmlChangeDelegate XmlChangeEvent;
		private CustomBandObject main;
		private Common oCommon;

		public PostThread(CustomBandObject main, Common oCommon)
		{
			this.main = main;
			this.oCommon = oCommon;
			main.ActionEvent +=new CustomBandObjectLib.CustomBandObject.ActionDelegate(main_ActionEvent);
			main.SpecialFileDownloadEvent +=new CustomBandObjectLib.CustomBandObject.SpecialFileDownloadDelegate(main_SpecialFileDownloadEvent);
			ThreadPool.QueueUserWorkItem(new WaitCallback(StartThread));
		}

		public void StartThread(object state)
		{
			if (oCommon.Reg.GetServer() != "")
			{
				try
				{
					while (main.IE.IsAdvised())
					{
						RefreshDownloads(null);	
						Thread.Sleep((int.Parse(oCommon.Reg.GetRefresh())) * 1000);					
					}
					Thread.CurrentThread.Abort();
				
				}
				catch(Exception e)
				{
					this.oCommon.Logger.WriteLog(e.Message, Log.LogType.Error);
				}
			}else{
				this.oCommon.Logger.WriteLog("Please, fill out the settings tab form and then restart Internet Explorer.", Log.LogType.Information);
			}
		}

		private Stream DownloadTorrentRequest(string url)
		{	
			try
			{
				HttpWebRequest request = (HttpWebRequest) WebRequest.Create(url);
				if(this.oCommon.FavHandler.GetFavoriteCookie(new Uri(url).Host))
				{
					request.CookieContainer = Common.GetUriCookieContainer(new Uri(url));
				}
				WebResponse response = request.GetResponse();
				return response.GetResponseStream();
			}
			catch(Exception e)
			{
					throw new TorrentException(e.Message);
			}
		}

		private HttpWebRequest CreateRequest()
		{
			HttpWebRequest request = (HttpWebRequest) WebRequest.Create(@"http://"+oCommon.Reg.GetServer()+":"+oCommon.Reg.GetPort()+"/process.cgi");
			NetworkCredential myCredentials = new NetworkCredential(oCommon.Reg.GetLogin(),oCommon.Reg.GetPassword());
			request.Credentials = myCredentials;
			request.Method = "POST";
			return request;
		}

		private void CreateCommand(ref HttpWebRequest request, Commands command) 
		{
			Stream requestStream = request.GetRequestStream();			
			ExecCommand(ref requestStream,command, ref obj);	
			requestStream.Close();
		}

		private void GetResponse(ref HttpWebRequest request,Commands command)
		{
			WebResponse response = request.GetResponse();				
			ParseResponse(response.GetResponseStream(),command);
		}

		private void ParseResponse(Stream stream,Commands command)
		{
			obj = oCommon.Parser.DoParse(ref stream, command);
			stream.Close();
			if (obj.List != null)
			{
				XmlChangeEvent(obj.List);
			}
		}

		private void Send(Protocol.Commands command)
		{
			try
			{
				HttpWebRequest request = CreateRequest();
				CreateCommand(ref request, command);
				GetResponse(ref request, command);
			}
			catch(Exception e) { throw new TorrentException(e.Message); }
		}

		public void RefreshDownloads(object state)
		{
			try
			{
				Monitor.Enter(this);
				Send(Protocol.Commands.getSingleton);
				Send(Protocol.Commands.getDownloadManager);
				Send(Protocol.Commands.getDownloads);
			}
			catch(TorrentException e) 
			{ 
				this.oCommon.Logger.WriteLog(e.Message, Log.LogType.Error); 
			}
			finally 
			{	
				Monitor.Exit(this);
			}
		}

		public void Stop(object state)
		{
			try
			{
				Monitor.Enter(this);
				Send(Protocol.Commands.getDownloadManager);
				Send(Protocol.Commands.stop);
			}
			catch(TorrentException e) 
			{ 
				this.oCommon.Logger.WriteLog(e.Message, Log.LogType.Error); 
			}
			finally 
			{	
				Monitor.Exit(this);
				Thread.CurrentThread.Abort();
			}
		}

		public void Start(object state)
		{
			try
			{
				Monitor.Enter(this);
				Send(Protocol.Commands.getDownloadManager);
				Send(Protocol.Commands.start);
			}
			catch //(TorrentException e) 
			{ 
				Restart(state);
				//this.oCommon.Logger.WriteLog(e.Message, Log.LogType.Error); 
			}
			finally 
			{	
				Monitor.Exit(this);
				Thread.CurrentThread.Abort();
			}
		}

		public void Delete(object state)
		{
			try
			{
				Monitor.Enter(this);
				Send(Protocol.Commands.getDownloadManager);
				Send(Protocol.Commands.delete);
			}
			catch(TorrentException e) 
			{ 
				this.oCommon.Logger.WriteLog(e.Message, Log.LogType.Error); 
			}
			finally 
			{	
				Monitor.Exit(this);
				Thread.CurrentThread.Abort();
			}
		}

		public void Restart(object state)
		{
			try
			{
				Monitor.Enter(this);
				Send(Protocol.Commands.getDownloadManager);
				Send(Protocol.Commands.restart);
			}
			catch(TorrentException e) 
			{ 
				this.oCommon.Logger.WriteLog(e.Message, Log.LogType.Error); 
			}
			finally 
			{	
				Monitor.Exit(this);
				Thread.CurrentThread.Abort();
			}
		}

		public void UploadFile(object url)
		{	
			try
			{
				Monitor.Enter(this);
				Send(Protocol.Commands.getSingleton);
				obj.TorrentURL = url.ToString();
				Send(Protocol.Commands.getTorrentManager);
				Send(Protocol.Commands.getURLDownloader);
				Send(Protocol.Commands.download);
				Send(Protocol.Commands.getDownloadManager);
				Send(Protocol.Commands.addDownload);
			}
			catch(TorrentException e) 
			{ 
				this.oCommon.Logger.WriteLog(e.Message, Log.LogType.Error); 
			}
			finally 
			{	
				Monitor.Exit(this);
				Thread.CurrentThread.Abort();
			}
		}

		public void UploadTorrentFile(object url)
		{	
			try
			{
				Monitor.Enter(this);
				obj.Torrent = ParseTorrentStream(DownloadTorrentRequest(((string)url)));
				Send(Protocol.Commands.getSingleton);
				Send(Protocol.Commands.getTorrentManager);
				Send(Protocol.Commands.createFromBEncodedData);
				Send(Protocol.Commands.getDownloadManager);
				Send(Protocol.Commands.addDownload);
			}
			catch(TorrentException e) 
			{ 
				this.oCommon.Logger.WriteLog(e.Message, Log.LogType.Error); 
			}
			finally 
			{	
				Monitor.Exit(this);
				obj.Torrent = null;
				Thread.CurrentThread.Abort();
			}
		}

		private void main_SpecialFileDownloadEvent(string url)
		{
			if (this.oCommon.FavHandler.GetFavoriteDownLoad(new Uri(url).Host.ToString()))
			{
				this.oCommon.Logger.WriteLog("Downloading file "+ url, Log.LogType.Information);
				ThreadPool.QueueUserWorkItem(new WaitCallback(UploadTorrentFile),url);
			}
			else
			{
				this.oCommon.Logger.WriteLog("Sending url "+ url,Log.LogType.Information);
				ThreadPool.QueueUserWorkItem(new WaitCallback(UploadFile), url);
			}
		}

		private string ParseTorrentStream(Stream input)
		{
			BufferedStream bs = new BufferedStream(input);
			BinaryReader reader = new BinaryReader(bs);
			StringBuilder hexString = new StringBuilder();

			bool reading = true;
			try
			{
				while (reading)
				{
					hexString.Append(reader.ReadByte().ToString("X2"));
				}
			}
			catch//(System.IO.EndOfStreamException eose)
			{
				reading = false;
			}
			reader.Close();

			return hexString.ToString();
		}

		private void main_ActionEvent(int action, string objectID)
		{
			if (action != -1)
			{
				switch(action)
				{
					case 0:{obj.DLObjectID = objectID; ThreadPool.QueueUserWorkItem(new WaitCallback(Start)); break;}
					case 1:{obj.DLObjectID = objectID; ThreadPool.QueueUserWorkItem(new WaitCallback(Stop));break;}
					case 2:{obj.DLObjectID = objectID; ThreadPool.QueueUserWorkItem(new WaitCallback(Delete));break;}
					case 4:{ThreadPool.QueueUserWorkItem(new WaitCallback(RefreshDownloads));break;}
				}
			}
		}
	}
}
