using System;
using System.Xml;
using System.IO;

namespace CustomBandObjectLib
{
	/// <summary>
	/// Summary description for protocol.
	/// </summary>
	public class Protocol
	{
		public enum Commands
		{
			getDownloads = 1,
			getSingleton = 2,
			getDownloadManager = 3,
			getURLDownloader = 4,
			download = 5,
			addDownload = 6,
			getTorrentManager = 7,
			createFromBEncodedData = 8,
			start = 9,
			stop = 10,
			delete = 11,
			getDownload = 12,
			restart = 14
		}

		public Protocol()
		{
		}

		private XmlTextWriter GetXmlWriter(ref Stream stream)
		{
			return new XmlTextWriter(stream, System.Text.Encoding.GetEncoding("ISO-8859-1"));
		}

		public void ExecCommand(ref Stream stream, Commands command, ref ResponseObject obj)
		{
			XmlTextWriter writer = GetXmlWriter(ref stream);
			switch (command)
			{
				case Commands.getDownloads: { GetDownloads(ref writer, ref obj);break;}
				case Commands.getSingleton: { GetSingleton(ref writer);break;}
				case Commands.getDownloadManager: { GetDownloadManager(ref writer, ref obj);break;}
				case Commands.getURLDownloader: { GetURLDownloader(ref writer, ref obj);break;}
				case Commands.download: { Download(ref writer, ref obj);break;}
				case Commands.addDownload: { AddDownload(ref writer, ref obj);break;}
				case Commands.getTorrentManager: { GetTorrentManager(ref writer, ref obj);break;}
				case Commands.createFromBEncodedData: { CreateFromBEncodedData(ref writer, ref obj);break;}
				case Commands.start: { Start(ref writer, ref obj);break;}
				case Commands.stop: { Stop(ref writer, ref obj);break;}
				case Commands.delete: { Delete(ref writer, ref obj);break;}
				case Commands.restart: { Restart(ref writer, ref obj);break;}
			}
			writer.Close();
		}

		private void Start(ref XmlTextWriter writer, ref ResponseObject obj)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.DLObjectID);
			writer.WriteEndElement();
			writer.WriteElementString("METHOD","start");
			writer.WriteElementString("CONNECTION_ID",obj.ConnectionID);
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();			
		}

		private void Stop(ref XmlTextWriter writer, ref ResponseObject obj)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.DLObjectID);
			writer.WriteEndElement();
			writer.WriteElementString("METHOD","stop");
			writer.WriteElementString("CONNECTION_ID",obj.ConnectionID);
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();			
		}

		private void Delete(ref XmlTextWriter writer, ref ResponseObject obj)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.DLObjectID);
			writer.WriteEndElement();
			writer.WriteElementString("METHOD","remove");
			writer.WriteElementString("CONNECTION_ID",obj.ConnectionID);
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();			
		}

		private void Restart(ref XmlTextWriter writer, ref ResponseObject obj)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.DLObjectID);
			writer.WriteEndElement();
			writer.WriteElementString("METHOD","restart");
			writer.WriteElementString("CONNECTION_ID",obj.ConnectionID);
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();			
		}

		private void GetDownloads(ref XmlTextWriter writer, ref ResponseObject obj)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.DLMObjectID);
			writer.WriteEndElement();
			writer.WriteElementString("METHOD","getDownloads");
			writer.WriteElementString("CONNECTION_ID",obj.ConnectionID);
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();			
		}
		
		private void CreateFromBEncodedData(ref XmlTextWriter writer, ref ResponseObject obj)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.TMObjectID);
			writer.WriteEndElement();
			writer.WriteElementString("METHOD","createFromBEncodedData[byte[]]");
			writer.WriteStartElement("PARAMS");
			writer.WriteElementString("ENTRY", obj.Torrent);
			writer.WriteEndElement();
			writer.WriteElementString("CONNECTION_ID",obj.ConnectionID);
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();
		}
	
		private void GetTorrentManager(ref XmlTextWriter writer, ref ResponseObject obj)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.SObjectID);
			writer.WriteEndElement();
			writer.WriteElementString("METHOD","getTorrentManager");
			writer.WriteElementString("CONNECTION_ID",obj.ConnectionID);
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();		
		}

		private void GetURLDownloader(ref XmlTextWriter writer, ref ResponseObject obj)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.TMObjectID);
			writer.WriteEndElement();
			writer.WriteElementString("METHOD","getURLDownloader[URL]");
			writer.WriteStartElement("PARAMS");
			writer.WriteElementString("ENTRY",obj.TorrentURL);
			writer.WriteEndElement();
			writer.WriteElementString("CONNECTION_ID",obj.ConnectionID );
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();
		}

		private void Download(ref XmlTextWriter writer, ref ResponseObject obj)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.TDObjectID);
			writer.WriteEndElement();
			writer.WriteElementString("METHOD","download");
			writer.WriteElementString("CONNECTION_ID",obj.ConnectionID );
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();
		}

		private void AddDownload(ref XmlTextWriter writer, ref ResponseObject obj)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.DLMObjectID);
			writer.WriteEndElement();
			writer.WriteElementString("METHOD","addDownload[Torrent]");
			writer.WriteStartElement("PARAMS");
			writer.WriteStartElement("ENTRY");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.DObjectID);
			writer.WriteEndElement();
			writer.WriteEndElement();
			writer.WriteEndElement();
			writer.WriteElementString("CONNECTION_ID",obj.ConnectionID );
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();
		}

		private void GetDownloadManager(ref XmlTextWriter writer, ref ResponseObject obj)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteStartElement("OBJECT");
			writer.WriteElementString("_object_id",obj.SObjectID);
			writer.WriteEndElement();
			writer.WriteElementString("METHOD","getDownloadManager");
			writer.WriteElementString("CONNECTION_ID",obj.ConnectionID );
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();
		}

		private void GetSingleton(ref XmlTextWriter writer)
		{
			writer.WriteStartDocument();
			writer.WriteStartElement("REQUEST");
			writer.WriteElementString("METHOD","getSingleton");
			writer.WriteElementString("REQUEST_ID",IDGenerator());
			writer.WriteEndElement();
			writer.WriteEndDocument();	
		}
	
		private String IDGenerator()
		{
			return Guid.NewGuid().GetHashCode().ToString();
		}
	}
}
