using System;
using System.Xml;
using System.IO;
using System.Text;

namespace CustomBandObjectLib
{
	/// <summary>
	/// Summary description for XMLReader.
	/// </summary>
	public class XMLReader: XmlTextReader
	{
		//private Log Logger;

		public XMLReader( ref ResponseObject obj, ref Stream stream, Protocol.Commands command):base(stream)
		{
			ReadElementString(ref obj, command);
		}

		public void ReadElementString(ref ResponseObject obj, Protocol.Commands command)
		{
			while (this.Read())
			{
				if (this.NodeType == XmlNodeType.Element)
				{
					switch (this.Name)
					{		
						case "_connection_id" : {obj.ConnectionID = this.ReadString();break;}
						case "_object_id" : { SetObjectID(ref obj,command);break;}
						case "hash": {obj.Hash = this.ReadString();break;}
						case "ERROR" : {throw new TorrentException(TrimString(this.ReadInnerXml()));}
						case "root": {
							while(this.Read())
							{
								switch(this.Name)
								{
									case "torrent" : 
									case "stats": 
									case "announce_result": 
									case "scrape_result": 
									{
										string name = this.Name;
										Stream s = GetStream("<root_" +name+ ">" + this.ReadInnerXml() + "</root_" +name+ ">");
										XMLReader r = new XMLReader(ref obj, ref s, command);
										r = null;
										s.Close(); 
										;break;
									}
									case "_object_id": {WriteString(GetNodeName(this.Name),GetFormat(this.Name,this.ReadInnerXml()), ref obj, false);break;}	
								}
							}
							return;
						
						}

						case "ENTRY" : 
						{
							obj.List = obj.List +"<table>";
							Stream s = GetStream("<root>" + this.ReadInnerXml() + "</root>");
							XMLReader r = new XMLReader(ref obj, ref s, command);
							r = null;
							obj.List = obj.List +"</table>";
							s.Close();
							;break;
						}

						case "root_torrent":
						{
							while(this.Read())
							{
								switch(this.Name)
								{
									case "name": {WriteString(GetNodeName(this.Name),GetFormat(this.Name,this.ReadInnerXml()), ref obj, true);break;}
									case "size": {WriteString(GetNodeName(this.Name),GetFormat(this.Name,this.ReadInnerXml()), ref obj, true);break;}
								}
							}
							return;
						}

						case "root_stats":
						{
							while(this.Read())
							{
								switch(this.Name)
								{
									case "downloaded": {WriteString(GetNodeName(this.Name),GetFormat(this.Name,this.ReadInnerXml()), ref obj, true);break;}
									case "uploaded": {WriteString(GetNodeName(this.Name),GetFormat(this.Name,this.ReadInnerXml()), ref obj, true);break;}
									case "status": {WriteString(GetNodeName(this.Name),GetFormat(this.Name,this.ReadInnerXml()), ref obj, true);break;}
									case "eta": {WriteString(GetNodeName(this.Name),GetFormat(this.Name,this.ReadInnerXml()), ref obj, true);break;}
									case "share_ratio": {WriteString(GetNodeName(this.Name),GetFormat(this.Name,this.ReadInnerXml()), ref obj, true);break;}
									case "download_average": {WriteString(GetNodeName(this.Name),GetFormat(this.Name,this.ReadInnerXml()), ref obj, true);break;}
									case "upload_average": {WriteString(GetNodeName(this.Name),GetFormat(this.Name,this.ReadInnerXml()), ref obj, true);break;}
									case "health": {WriteString(GetNodeName(this.Name),this.ReadInnerXml(), ref obj, false);break;}
								}
							}
							return;
						}

						case "root_announce_result":{ return;}
						case "root_scrape_result":{ return;}
					}	
				}
			}
		}

		public Stream GetStream(string xml)
		{
			Stream stream = new MemoryStream();
			StreamWriter writer = new StreamWriter(stream);
			writer.Write(xml);
			writer.Flush();				
			stream.Position = 0;
			return stream;
		}

		public void SetObjectID(ref ResponseObject obj, Protocol.Commands command)
		{
			switch(command)
			{
				case Protocol.Commands.getSingleton: {obj.SObjectID = this.ReadString();break;}
				case Protocol.Commands.getDownloadManager: {obj.DLMObjectID = this.ReadString();break;}
				case Protocol.Commands.getTorrentManager: {obj.TMObjectID = this.ReadString();break;}
				case Protocol.Commands.download: {obj.DObjectID = this.ReadString();break;}
				case Protocol.Commands.getURLDownloader: {obj.TDObjectID = this.ReadString();break;}				
				case Protocol.Commands.createFromBEncodedData: {obj.DObjectID = this.ReadString();break;}
			}
		}

		public string TrimString(string entry)
		{
			char[] trimArray = new char[1];
			trimArray[0] = ' ';
			string rough = entry.Replace("\r\n","");
			rough = rough.TrimEnd(trimArray);
			rough = rough.TrimStart(trimArray);
			return rough;
		}
	
		public void WriteString(string name, string entry, ref ResponseObject obj, bool showInGrid)
		{
			StringBuilder b = new StringBuilder(obj.List);
			if (showInGrid)
			{
				b.Append("<" + name +" show=\"1\">");
			}
			else
			{
				b.Append("<" + name +" show=\"0\">");
			}

			if (entry == null) entry = "Torrent removed";
			b.Append(entry);
			b.Append("</" +name +">");
			obj.List = b.ToString();
		}

		private string GetNodeName(string name)
		{
			switch(name)
			{
				case "name":{ return "Name";}
				case "size": {return "Size";}
				case "downloaded": { return "Downloaded";}
				case "uploaded": { return "Uploaded";}
				case "status": { return "Status";}
				case "eta": { return "Left";}
				case "share_ratio": { return "Ratio";}
				case "download_average": { return "Download";}
				case "upload_average": { return "Upload";} 
				case "_object_id": { return "ObjectID";}
				case "health": { return "Health";}
				default : return "";
			}
		}

		private string GetFormat(string name, string entry)
		{
			switch(name)
			{
				case "name":{ return TrimString(entry);}	
				case "size": {return new StringBuilder(string.Empty).AppendFormat("{0:0.##}",decimal.Parse(TrimString(entry)) /1000000).ToString() +" MB";}
				case "downloaded": { return new StringBuilder(string.Empty).AppendFormat("{0:0.##}",decimal.Parse(TrimString(entry)) /1000000).ToString() +" MB";}
				case "uploaded": { return new StringBuilder(string.Empty).AppendFormat("{0:0.##}",decimal.Parse(TrimString(entry)) /1000000).ToString() +" MB";}
				case "status": { return TrimString(entry);}
				case "eta": { return TrimString(entry);}
				case "share_ratio": { return new StringBuilder(string.Empty).AppendFormat("{0:0.000}",decimal.Parse(TrimString(entry)) /1000).ToString();}
				case "download_average": { return new StringBuilder(string.Empty).AppendFormat("{0:0.##}",decimal.Parse(TrimString(entry)) /1000).ToString() +" KB/s";}
				case "upload_average": { return new StringBuilder(string.Empty).AppendFormat("{0:0.##}",decimal.Parse(TrimString(entry)) /1000).ToString() +" KB/s";}
				case "_object_id": { return new StringBuilder(TrimString(entry)).ToString();}
				default : return "";
			}
		}
	}
}
