using System;
using System.Drawing;
using System.Windows.Forms;
using System.Xml;
using System.Reflection;
using System.IO;
using System.Runtime.InteropServices;

[assembly: AssemblyVersion("1.0.0.0")]
[assembly: AssemblyKeyFile(@"..\..\ListView.snk")]

namespace myListView
{		

	/// <summary>
	/// Summary description for myListView.
	/// </summary>
	[ToolboxBitmap(typeof(System.Windows.Forms.ListView))]
	public class myListView : System.Windows.Forms.ListView
	{
		private const int WM_SETREDRAW      = 0x000B;
		private const int WM_USER           = 0x400;
		private const int EM_GETEVENTMASK   = (WM_USER + 59);
		private const int EM_SETEVENTMASK   = (WM_USER + 69);

		[DllImport("user32", CharSet = CharSet.Auto)]
		public static extern int SendMessage(IntPtr handle, int messg, int wparam, int lparam);

		bool firstPass = true;
		private System.Windows.Forms.ImageList imageList1;
		private System.ComponentModel.IContainer components;
		int selectedItem = -1;

		public myListView()
		{
			InitializeComponent();
			this.SetStyle(System.Windows.Forms.ControlStyles.DoubleBuffer, true);
			this.SetStyle(System.Windows.Forms.ControlStyles.ResizeRedraw, true);
		}

		public int GetSelectedItem()
		{
			return selectedItem;
		}

		public void ReadXML(string xml)
		{
			int eventMask = 0;
			try
			{
				SendMessage(this.Handle, WM_SETREDRAW, 0, 0);
				eventMask = SendMessage(this.Handle, EM_GETEVENTMASK, 0, 0);
				ParseXML(xml);
			}
			finally
			{
				SendMessage(this.Handle, EM_SETEVENTMASK, 0, eventMask);
				SendMessage(this.Handle, WM_SETREDRAW, 1, 0);		
				
			}	
		}

		public void ParseXML(string xml)
		{
			ListViewItem item = null;
			bool firstNode = true;
			this.Items.Clear();
			XmlTextReader xmlReader = new XmlTextReader(GetStream(xml));
			while(xmlReader.Read())
			{
				switch (xmlReader.NodeType)
				{
					case XmlNodeType.Element:
					
					switch(xmlReader.Name)
					{	
						case "table": 
						{
							firstNode=true;
							while(xmlReader.Read())
							{
								if (xmlReader.NodeType == XmlNodeType.EndElement)
								{
									if (xmlReader.Name =="table")
									{
										if(firstPass) firstPass = !firstPass;
										break;
									}
								}
								if (firstPass)
								{
									if (xmlReader.GetAttribute("show") == "1")
									{		
										this.Columns.Add(xmlReader.Name,100, HorizontalAlignment.Right);
									}
									else
									{
										ColumnHeader ch = this.Columns.Add(xmlReader.Name,100, HorizontalAlignment.Right);
										this.Columns[ch.Index].Width = 0;
									}
								}

								if(firstNode)
								{
									if (xmlReader.GetAttribute("show") == "1")
									{
										firstNode = false;
										item = this.Items.Add(xmlReader.ReadString());
									}
									else
									{
										firstNode = false;
										item = this.Items.Add(xmlReader.ReadString());
										this.Columns[item.Index].Width = 0;
									}
								}
								else
								{
									if (xmlReader.GetAttribute("show") == "1")
									{
										firstNode = false;
										item.SubItems.Add(xmlReader.ReadString());
									}
									else
									{
										if (xmlReader.Name == "Health")
										{
											item.ImageIndex = int.Parse(xmlReader.ReadString()) -1;
										}
										else
										{
											firstNode = false;
											ListViewItem.ListViewSubItem subItem;
											subItem = item.SubItems.Add(xmlReader.ReadString());
											this.Columns[item.SubItems.IndexOf(subItem)].Width = 0;	
										}
									}							
								}
							}
						;break;
						}
					}
				;break;
				}
			}
			if (selectedItem != -1)
			{
				this.Items[selectedItem].Selected = true;
			}
			
		}

		public Stream GetStream(string xml)
		{
			Stream stream = new MemoryStream();
			StreamWriter writer = new StreamWriter(stream,System.Text.Encoding.GetEncoding("ISO-8859-1"));
			writer.Write(xml);
			writer.Flush();				
			stream.Position = 0;
			return stream;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}

		#region Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(myListView));
			this.imageList1 = new System.Windows.Forms.ImageList(this.components);
			// 
			// imageList1
			// 
			this.imageList1.ColorDepth = System.Windows.Forms.ColorDepth.Depth32Bit;
			this.imageList1.ImageSize = new System.Drawing.Size(17, 17);
			this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
			this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// myListView
			// 
			this.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.FullRowSelect = true;
			this.GridLines = true;
			this.SmallImageList = this.imageList1;
			this.View = System.Windows.Forms.View.Details;
			this.SelectedIndexChanged += new System.EventHandler(this.myListView_SelectedIndexChanged);

		}

		#endregion


		protected override void OnPaintBackground(PaintEventArgs pevent)
		{
			base.OnPaintBackground (pevent);
			int borderWidth = 1; 
			
			Color borderColor = Color.Black; 
			System.Drawing.Pen borderPen = new System.Drawing.Pen(borderColor,borderWidth);
			pevent.Graphics.DrawRectangle(borderPen, pevent.ClipRectangle);
			borderPen.Dispose();
		}

 
		private void myListView_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			try 
			{
				selectedItem = this.SelectedIndices[0];
			}
			catch { selectedItem = -1;}
		}
	}
}
